function [H1,H2,F1,F2] = solve_coop(A0,A1,A2,A3,A4,A5,W,Q,H1,F1,beta,t,varargin)

dampen = 0;

if rem(nargin,13) == 0
    verbose = varargin{1};
end
if exist('verbose','var')==0;
   verbose= 0; 
end

% solves for optimal discretionary policy in the system: 
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X1_t + A4*X1_{t+1} +A5*V_t
% 
% where: Y is non policy variable
%        X is policy variables
%        V is pure i.i.d. shocks
% 
% policymaker objective given by matrices are W and Q, see Dennis 2007
%
% Solution is of the form:  Y_t = H1*Y_{t-1} + H2*V_t
%                           X_t = F1*Y_{t-1} + F2*V_t
%
%
% INPUT: A0,A1,A2,A3,A4,A5   -   matrices from equilibrium conditions
%        W                   -   loss matrix on nonpolicy variables       
%        Q                   -   loss matrix on policy variables
%        H1,F1               -   starting guesses for solution
%        beta                -   discount factor
%        t                   -   scalar stopping criterion for iteration
%        
% OUTPUT: H1,H2,F1,F2        -   solution matrices


if size(A0,1)~=size(A0,2); fprintf('\n You must have as many equations as nonpolicy variables \n'); return; end;
if rank(A0) ~= size(A0,2); fprintf('\n Warning your A0 matrix is singular \n'); return; end;

c    = 0;
cmax = 10^6;
diff = 999;

while diff > t
   
  if c > cmax; break; fprintf('\n No convergence after %g iterations \n', cmax); return; end;
   
  c = c + 1;
  H1_old  = H1;
  F1_old  = F1;  
% Step 2: solve for D and P
  D = A0 - A2*H1 - A4*F1;
  P = doublej_mp(beta*H1,W+beta*transpose(F1)*Q*F1,H1);
  check = - P + W + beta*transpose(F1)*Q*F1 + beta*transpose(H1)*P*H1;

  % Step 3: Update
% rank( Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )
% size( Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 );


  F1 = (1-dampen)*(-inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A1) + dampen*F1_old;  
  H1 = inv(D)*(A1+A3*F1);
  
  
 if ~all(abs(eig(H1))< 1.00001) == 1; 
            fprintf('\n Explosive eigenvalues: better introduce dampening ... \n'); 
%            break;
%            H1 = beta^2*inv(max(max(abs(eig(H1)))))*H1;
  end;
 diff = max([ max(max(abs(H1-H1_old))) ,  max(max(abs(F1-F1_old))) ]); 
 
   
%      colormap(jet);
%      colorbar;
%      subplot(1,2,1);
%      imagesc(abs(H1-H1_old)); colorbar;  title('H1');
%      subplot(1,2,2);
%      imagesc(abs(F1-F1_old)); colorbar;  title('F1');
%      pause(1/100000);
end;  

F2 = -inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A5; 
H2 = inv(D)*(A5+A3*F2);

check_1 = -((A0 - A2*H1 -A4*F1)*H1 -A3*F1) + A1;
check_2 = -((A0 - A2*H1)*H2 -A3*F2 - A4*F1*H2) + A5;

check_3 = A3'*(D')^(-1)*P*D^(-1)*A1 + (Q+A3'*(D')^(-1)*P*D^(-1)*A3)*F1;
check_4 = A3'*(D')^(-1)*P*D^(-1)*A5 + (Q+A3'*(D')^(-1)*P*D^(-1)*A3)*F2;

% save FOC_coop check_3 check_4

if verbose == 1
  fprintf('\n Convergence after %g iterations \n', c);  
  fprintf('\n Accuracy check for solving functional equation %g and %g \n', max(max(abs(check_1))), max(max(abs(check_2))),max(max(abs(check_3))) , max(max(abs(check_4)))   );
end

  