function [A,B] = solve_cooperation_commitment(A0,A1,A2,A3,A4,A5,W,Q,beta)


% Constraints for policy maker (with multipliers lambda)
% 0 = A0*Y_{t} - A1*Y_{t-1} - A2*Y_{t+1} - A3*X1_t - A4*X1_{t+1} -A5*V_t)

% objectives:  discounted sums of y_t'W*y_t + x1_t'*Q*x1_t 

% set of FOC is in matrix notation
% --------------------------------------------
% 0 = F*Z_{t+1} + C*Z_t + L*Z_{t-1} + S*shock_t
% where Z' = [y',x',lambda'];

% ORDER OF EQUATIONS IN SYSTEM
% ------------------------------------
% FOC y_t 
% 0 = W*y_t + A0'*lambda_t - beta*A1'*lambda_{t+1} - 1/beta*A2'*lambda_{t-1}
% FOC x_t 
% 0 = Q*x_t - A3'*lambda{t} - 1/beta*A4'*lambda{t-1}
% constraints
% 0 = A0*Y_{t} - A1*Y_{t-1} - A2*Y_{t+1} - A3*X_t - A4*X_{t+1} -A5*V_t)


[ny,nx] = size(A3);
ns      = size(A5,2);

% coefficients on future Z
F = [ zeros(ny,ny),  zeros(ny,nx),   -beta*A1',  ; ...
      zeros(nx,ny),  zeros(nx,nx),   zeros(nx,ny); ...
      -A2         ,  -A4,            zeros(ny,ny)];
  
% coefficients on current Z
C = [ W,             zeros(ny,nx),    A0';        ...
      zeros(nx,ny), Q            ,  -A3';        ...
      A0         ,  -A3,             zeros(ny,ny)];
  
% coefficients on lagged Z  
L =  [zeros(ny,ny), zeros(ny,nx), -1/beta*A2';  ...
      zeros(nx,ny), zeros(nx,nx), -1/beta*A4';  ...
      -A1         , zeros(ny,nx), zeros(ny,ny) ];

% coefficients on shocks  
S =  [  zeros(ny,ns);  ...
        zeros(nx,ns); ... 
       -A5 ];         
   
dim = 2*ny + nx;

% Now put terminal system into gensys format to solve for equilibrium
g0  = [[-C, -F];[eye(dim), zeros(dim)]];         % coefficients on current period variables (including expectations of tomorrows variables as separate variables)
g1	= [[L,zeros(dim)];[zeros(dim),eye(dim)]];    % coefficients on last period variables
c   = zeros(2*dim,1);                            % constant (not present in terminal regime)
psi = [S;zeros(dim,ns)];                         % coefficients on structural shocks
pie = [zeros(dim);eye(dim)];                     % coefficients on expectational errors
div = 1.00001;                                         % cutoff between explosive and non-explosive roots

[NA,c,NB,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,c,psi,pie,div);

% Extract transition matrix (discard expectational variables that are of no interest here)
A = NA(1:dim,1:dim);
B = NB(1:dim,1:ns);


%max(max(abs(F*A^2 + C*A + L)))

max_eig = max(abs(eig(A)));
%fprintf('\n Maximum eigenvalue of the system = %g \n',max_eig);

