function [H1,H2,F1,F1T,F2,F2T,G] = solve_leader(A0,A1,A2,A3,A3T,A4,A4T,A5,W,WT,Q,QT,H1,F1,F1T,G,beta,t);

global dampen

% solves for optimal discretionary policy in the system: 
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X1_t + A4*X1_{t+1} + A3T*X2_t + A4T*X2_{t+1} +A5*V_t
% 
% where: Y  is non policy variable
%        X1 is policy variable of policymaker 1, the LEADER
%        X2 is policy variable of policymaker 2, the FOLLOWER
%        V is pure i.i.d. shocks
% 
% ASSUMPTION: X1 is chosen before X2 within the period
%
% policymaker 1 objective given by matrices are W and Q, see Dennis 2007
% policymaker 2 objective given by matrices are WT and QT
%
% Solution is of the form:  Y_t  = H1*Y_{t-1}  + H2*V_t
%                           X1_t = F1*Y_{t-1}  + F2*V_t
%                           X2_t = F1T*Y_{t-1} + G*X1_t + F2T*V_t
%
% INPUT: A0,A1,A2,A3,A4,A3T,A4T,A5   -   matrices from equilibrium conditions
%        W,WT                        -   loss matrix on nonpolicy variables
%        Q,QT                        -   loss matrix on policy variables
%        H1,F1,F1T,G                 -   starting guesses for solution
%        beta                        -   discount factor
%        t                           -   scalar stopping criterion for iteration
%        
% OUTPUT: H1,H2,F1,F1T,F2,F2T,G      -   solution matrices


if size(A0,1)~=size(A0,2); fprintf('\n You must have as many equations as non policy variables \n'); return; end;
if rank(A0) ~= size(A0,2); fprintf('\n Warning your A0 matrix is singular \n'); return; end;


c = 0;
cmax = 10^6;
diff = 999;

while diff > t; 

  if c > cmax; break; fprintf('\n No convergence after %g iterations \n', cmax); return; end;
  
  c = c + 1;
  G_old   = G;
  H1_old  = H1;
  F1_old  = F1;
  F1T_old = F1T;
  
  
% Step 2: solve for D and P
  D = A0 - A2*H1 - A4*F1 - A4T*F1T -A4T*G*F1;
  P = doublej_mp(beta*H1,W+beta*transpose(F1)*Q*F1,H1);
% check = - P + W + beta*transpose(F1)*Q*F1 + beta*transpose(H1)*P*H1
  PT = doublej_mp(beta*H1,WT+beta*transpose(F1T+G*F1)*QT*(F1T+G*F1),H1);
% check = - PT + WT + beta*transpose(F1T+G*F1)*QT*(F1T+G*F1) + beta*transpose(H1)*PT*H1  
% Step 3: Update
 temp0 = G;
 temp1 = F1T;
 temp2 = F1;
 
 G   = (1-dampen)*(-inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3) + dampen*temp0;
 F1T = (1-dampen)*(-inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A1) + dampen*temp1;
 F1  = (1-dampen)*(-inv(Q +transpose(A3+A3T*G)*inv(transpose(D))*P*inv(D )*(A3+A3T*G) )*transpose(A3+A3T*G)*inv(transpose(D))*P*inv(D)*( A1+A3T*F1T)) + dampen*temp2; 
 
 H1  =  inv(D)*(A1+A3*F1+A3T*F1T+A3T*G*F1);
  
 if ~all(abs(eig(H1))< 1) == 1; 
%            fprintf('\n Explosive eigenvalues: better introduce dampening ... \n'); 
%            break;
%            H1 = beta^2*inv(max(max(abs(eig(H1)))))*H1;
  end;  
  
 diff = max( [max(max(abs(H1-H1_old))) , max(max(abs(F1-F1_old))) , max(max(abs(F1T-F1T_old))) , max(max(abs(G-G_old))) ] );    
 
%    
%     colormap(jet);
%     colorbar;
%     subplot(1,4,1);
%     imagesc(abs(H1-H1_old)); colorbar;  title('H1');
%     subplot(1,4,2);
%     imagesc(abs(F1-F1_old)); colorbar;  title('F1');
%     subplot(1,4,3);
%     imagesc(abs(F1T-F1T_old)); colorbar;  title('F1T');
%     subplot(1,4,4);
%     imagesc(abs(G-G_old)); colorbar;  title('G');
%     
%     pause(1/100000);
  
end;  

D  = A0 - A2*H1 - A4*F1 - A4T*F1T -A4T*G*F1;

F2T = -inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A5;
F2  = -inv(Q +transpose(A3 +A3T*G)*inv(transpose(D))*P*inv(D )*(A3+A3T*G) )*transpose(A3 +A3T*G)*inv(transpose(D))*P*inv(D)*(A5+A3T*F2T); 
H2  =  inv(D)*(A5 + A3*F2 + A3T*F2T + A3T*G*F2);


% double check FOC

D  = A0 - A2*H1 - A4*F1 - A4T*F1T -A4T*G*F1;
P  = doublej_mp(beta*H1,W+beta*transpose(F1)*Q*F1,H1);
PT = doublej_mp(beta*H1,WT+beta*transpose(F1T+G*F1)*QT*(F1T+G*F1),H1);

% A check that private sector FOC are satisfied
% coefficients on Y_0 should be zero
check_1 = -D*H1 + A1 + A3*F1 + A3T*F1T + A3T*G*F1;
% coefficient on shock should be zero
check_2 = -D*H2      + A3*F2 + A3T*F2T + A3T*G*F2 + A5;
% B check that player one FOC are satisfied
check_3 = (A3+A3T*G)'*(D')^(-1)*P*D^(-1)*(A1+A3T*F1T) + (Q+(A3+A3T*G)'*(D')^(-1)*P*D^(-1)*(A3+A3T*G))*F1;
check_4 = (A3+A3T*G)'*(D')^(-1)*P*D^(-1)*(A5+A3T*F2T) + (Q+(A3+A3T*G)'*(D')^(-1)*P*D^(-1)*(A3+A3T*G))*F2;
% C check that player two FOC are satisfied
check_5 = A3T'*(D')^(-1)*PT*D^(-1)*(A1 + A3*F1) + (QT+A3T'*(D')^(-1)*PT*D^(-1)*A3T)*(F1T + G*F1);
check_6 = A3T'*(D')^(-1)*PT*D^(-1)*(A5 + A3*F2) + (QT+A3T'*(D')^(-1)*PT*D^(-1)*A3T)*(F2T + G*F2);


