function [H1,H2,F1,F2,F1T,F2T] = solve_nash(A0,A1,A2,A3,A3T,A4,A4T,A5,W,WT,Q,QT,H1,F1,F1T,beta,t,varargin)

global solution_algo dampen dampen1 dampen2 show_iter

if exist('dampen','var')==0;
   dampen = 0; 
end
if rem(nargin,18) == 0
    verbose = varargin{1};
end

if exist('verbose','var')==0;
   verbose = 0; 
end


% solves for optimal discretionary policy in the system: 
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X1_t + A4*X1_{t+1} + A3T*X2_t + A4T*X2_{t+1} +A5*V_t
% 
% where: Y  is non policy variable
%        X1 is policy variable of policymaker 1
%        X2 is policy variable of policymaker 2
%        V is pure i.i.d. shocks
% 
% ASSUMPTION: X1 and X2 are chosen simultaneously
%
% policymaker 1 objective given by matrices are W and Q, see Dennis 2007
% policymaker 2 objective given by matrices are WT and QT
%
% Solution is of the form:  Y_t  = H1*Y_{t-1}  + H2*V_t
%                           X1_t = F1*Y_{t-1}  + F2*V_t
%                           X2_t = F1T*Y_{t-1} + F2T*V_t
%
% INPUT: A0,A1,A2,A3,A4,A3T,A4T,A5   -   matrices from equilibrium conditions
%        W,WT                        -   loss matrix on nonpolicy variables
%        Q,QT                        -   loss matrix on policy variables
%        H1,F1,F1T                   -   starting guesses for solution
%        beta                        -   discount factor
%        t                           -   scalar stopping criterion for iteration
%        
% OUTPUT: H1,H2,F1,F1T,F2,F2T        -   solution matrices



if size(A0,1)~=size(A0,2); fprintf('\n You must have as many equations as non policy variables \n'); return; end;
if rank(A0) ~= size(A0,2); fprintf('\n Warning your A0 matrix is singular \n'); return; end;
c    = 0;
cmax = 10^6;
diff = 999;

while diff > t; 
  
  if c > cmax; break; fprintf('\n No convergence after %g iterations \n', cmax); return; end;
  
  c = c + 1;
  H1_old  = H1;
  F1_old  = F1;
  F1T_old = F1T;
 
% Step 2: solve for D and P
  D = A0 - A2*H1 - A4*F1 - A4T*F1T;
  
  P = doublej_mp(beta*H1,W+beta*transpose(F1)*Q*F1,H1);
%  check = - P + W + beta*transpose(F1)*Q*F1 + beta*transpose(H1)*P*H1

%  if (P*inv(D)*A3) == zeros(size((P*inv(D)*A3)));
%     fprintf(' \n P*inverse(D)*A3 is null matrix \n');
%  end;
      
  PT = doublej_mp(beta*H1,WT+beta*transpose(F1T)*QT*F1T,H1);
  
  
  
  if (PT*inv(D)*A3T) == zeros(size((PT*inv(D)*A3T)));
     fprintf('\n PT*inverse(D)*A3T is null matrix \n');
  end;
  
% check = - PT + WT + beta*transpose(F1T)*QT*F1T + beta*transpose(H1)*PT*H1  
% Step 3: Update
% can solve this simultaneously ...

if solution_algo == 1
  constant_1 = -inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A1;
  constant_2 = -inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A1;
  theta_1    = -inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A3T; 
  theta_2    = -inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3;
  LHS        = [ eye(size(F1,1)), - theta_1; -theta_2, eye(size(F1T,1))];
    
  temp1 = F1T;
  temp2 = F1;
  
   solution = inv(LHS)*[constant_1;constant_2];
   F1       = (1-dampen).*solution(1:size(F1T,1),:)     + dampen.*temp2;
   F1T      = (1-dampen).*solution(size(F1T,1)+1:end,:) + dampen.*temp1;
elseif solution_algo==2
       temp1 = F1T;
       temp2 = F1;
       F1T   = (1-dampen1).*(-inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*(A1+A3*F1  )) + dampen1.*temp1;
       F1    = (1-dampen2).*(-inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*( A1+A3T*F1T)) + dampen2.*temp2; 
elseif solution_algo==3
       temp1 = F1T;
       temp2 = F1;
       F1    = (1-dampen).*(-inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*( A1+A3T*F1T)) + dampen.*temp2; 
       F1T   = (1-dampen).*(-inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*(A1+A3*F1  )) + dampen.*temp1;
end
       % % % % %   
% 
  D   =   A0 - A2*H1 - A4*F1 - A4T*F1T;
  % H1  =  inv(D)*(A1 + A3*F1 + A3T*F1T);
  
  H1  =  (1-dampen).*inv(D)*(A1 + A3*F1 + A3T*F1T) + dampen.*H1_old;
  
  
  
% if ~all(abs(eig(H1))< 1) == 1; 
%             abs(eig(H1))
%            fprintf('\n Explosive eigenvalues: better introduce dampening ... \n'); 
%            break;
%           H1 = beta^2*inv(max(max(abs(eig(H1)))))*H1;
% end;
  
  diff = max( [max(max(abs(H1-H1_old))) , max(max(abs(F1-F1_old))), max(max(abs(F1T-F1T_old)))]  );
  
  if mod(c,1000)==0
      %fprintf('\n Residual after iteration %g is %2.10f \n',c,diff);
  end

  if show_iter==1
    colormap(jet);
    colorbar;
    subplot(1,3,1);
    imagesc(abs(H1-H1_old)); colorbar;  title('H1');
    subplot(1,3,2);
    imagesc(abs(F1-F1_old)); colorbar;  title('F1');
    subplot(1,3,3);
    imagesc(abs(F1T-F1T_old)); colorbar;  title('F1T');
    pause(1/100000);
  end
  
end;  

% can solve for F2 and F2T simultaneously ...
    
constant_1 = -inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A5;
theta_1    = -inv(Q +transpose(A3 )*inv(transpose(D))*P*inv(D )*A3 )*transpose(A3 )*inv(transpose(D))*P*inv(D)*A3T; 
theta_2    = -inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3;
constant_2 = -inv(QT+transpose(A3T)*inv(transpose(D))*PT*inv(D)*A3T)*transpose(A3T)*inv(transpose(D))*PT*inv(D)*A5;
LHS        = [ eye(size(A3,2)), - theta_1; -theta_2, eye(size(A3T,2))];


    
solution = inv(LHS)*[constant_1;constant_2];



F2       = solution(1:size(A3,2),:);
F2T      = solution(size(A3,2)+1:end,:);  

D = A0 - A2*H1 - A4*F1 - A4T*F1T;

H2       = inv(D)*(A5+A3*F2+A3T*F2T);




D  = A0 - A2*H1 - A4*F1 - A4T*F1T;
P  = doublej_mp(beta*H1,W+beta*transpose(F1)*Q*F1,H1);
PT = doublej_mp(beta*H1,WT+beta*transpose(F1T)*QT*F1T,H1);

% A check that private sector equilibrium conditions are satisfied
% coefficients on Y_0 should be zero
check_1 = -((A0 - A2*H1 -A4*F1 - A4T*F1T)*H1 -A3*F1 -A3T*F1T) + A1;
% coefficient on shock should be zero
check_2 = -((A0 - A2*H1)*H2 -A3*F2 -A3T*F2T - A4*F1*H2 - A4T*F1T*H2) + A5;

% B check that player one FOC are satisfied
check_3 = A3'*(D')^(-1)*P*D^(-1)*(A1+A3T*F1T) + (Q+A3'*(D')^(-1)*P*D^(-1)*A3)*F1;
check_4 = A3'*(D')^(-1)*P*D^(-1)*(A5+A3T*F2T) + (Q+A3'*(D')^(-1)*P*D^(-1)*A3)*F2;

% C check that player two FOC are satisfied
check_5 = A3T'*(D')^(-1)*PT*D^(-1)*(A1+A3*F1) + (QT+A3T'*(D')^(-1)*PT*D^(-1)*A3T)*F1T;
check_6 = A3T'*(D')^(-1)*PT*D^(-1)*(A5+A3*F2) + (QT+A3T'*(D')^(-1)*PT*D^(-1)*A3T)*F2T;

if verbose == 1
   %fprintf('Convergence after %g iterations', c);
   %fprintf('\n Accuracy check for solving functional equation %g, %g, %g, %g, %g, %g \n', max(max(abs(check_1))), max(max(abs(check_2))),max(max(abs(check_3))),max(max(abs(check_4))),max(max(abs(check_5))),max(max(abs(check_6))) );
end