function [A,B] = solve_open_loop(A0,A1,A2,A3,A3T,A4,A4T,A5,W,WT,Q,QT,beta)

% dbstop at 98

% each policy maker problem is
% ---------------------------------
% maximize your own loss function taken as given equilibrium BEHAVIOR of other policymaker (open loop)
% this is different from taking as given the REACTION FUNCTION of the other player as in proper Nash (closed loop)

% Constraints for each policy maker (with multipliers lambda1 and lambda2)
% 0 = A0*Y_{t} - A1*Y_{t-1} - A2*Y_{t+1} - A3*X1_t - A4*X1_{t+1} - A3T*X2_t - A4T*X2_{t+1} -A5*V_t)

% objectives:  discounted sums fo y_t'W*y_t + x1_t'*Q*x1_t for policy maker 1 and y_t'WT*y_t + x2_t'*QT*x2_t for 2

% set of FOC is in matrix notation
% --------------------------------------------
% 0 = F*Z_{t+1} + C*Z_t + L*Z_{t-1} + S*shock_t
% where Z' = [y',x1',x2',lambda1',lambda2'];

% ORDER OF EQUATIONS IN SYSTEM
% ------------------------------------
% FOC y_t for player 1
% 0 = W*y_t +A0'*lambda_1t -beta*A1'*lambda_1{t+1} - 1/beta*A2'*lambda_1{t-1}
% FOC y_t for player 2
% 0 = WT*y_t +A0'*lambda_2t -beta*A1'*lambda_2{t+1} - 1/beta*A2'*lambda_2{t-1}
% FOC x1_t for player 1
% 0 = Q*x1_t - A3'*lambda_1{t}   - 1/beta*A4'*lambda_1{t-1}
% FOC x2_t for player 2
% 0 = QT*x2_t - A3T'*lambda_2{t} - 1/beta*A4T'*lambda_2{t-1}
% constraints
% 0 = A0*Y_{t} - A1*Y_{t-1} - A2*Y_{t+1} - A3*X1_t - A4*X1_{t+1} - A3T*X2_t - A4T*X2_{t+1} -A5*V_t)

[ny,nx1] = size(A3);
 nx2     = size(A3T,2);
 ns      = size(A5,2);

% coefficients on future Z
F = [ zeros(ny,ny),  zeros(ny,nx1),  zeros(ny,nx2),  -beta*A1',     zeros(ny,ny);  ...
      zeros(ny,ny),  zeros(ny,nx1),  zeros(ny,nx2),  zeros(ny,ny),  -beta*A1';     ...
      zeros(nx1,ny), zeros(nx1,nx1), zeros(nx1,nx2), zeros(nx1,ny), zeros(nx1,ny); ...
      zeros(nx2,ny), zeros(nx2,nx1), zeros(nx2,nx2), zeros(nx2,ny), zeros(nx2,ny); ...
      -A2         ,  -A4,            -A4T,           zeros(ny,ny),  zeros(ny,ny) ];
  
% coefficients on current Z
C = [ W,             zeros(ny,nx1),  zeros(ny,nx2),  A0',           zeros(ny,ny);  ...
      WT,            zeros(ny,nx1),  zeros(ny,nx2),  zeros(ny,ny),  A0';           ...
      zeros(nx1,ny), Q            ,  zeros(nx1,nx2), -A3',          zeros(nx1,ny); ...
      zeros(nx2,ny), zeros(nx2,nx1), QT,             zeros(nx2,ny), -A3T';          ...
      A0         ,  -A3,             -A3T,           zeros(ny,ny),  zeros(ny,ny) ];
  
% coefficients on lagged Z  
L =  [zeros(ny,ny),  zeros(ny,nx1),  zeros(ny,nx2),  -1/beta*A2',   zeros(ny,ny);  ...
      zeros(ny,ny),  zeros(ny,nx1),  zeros(ny,nx2),  zeros(ny,ny),  -1/beta*A2';   ...
      zeros(nx1,ny), zeros(nx1,nx1), zeros(nx1,nx2), -1/beta*A4',   zeros(nx1,ny); ...
      zeros(nx2,ny), zeros(nx2,nx1), zeros(nx1,nx2), zeros(nx2,ny), -1/beta*A4T';   ...
      -A1          , zeros(ny,nx1),  zeros(ny,nx2),  zeros(ny,ny),  zeros(ny,ny) ];

% coefficients on shocks  
S =  [  zeros(ny,ns);  ...
        zeros(ny,ns);  ...   
        zeros(nx1,ns); ... 
        zeros(nx2,ns);  
       -A5 ];         
   
dim = 3*ny + nx1 + nx2;

% Now put terminal system into gensys format to solve for equilibrium
g0  = [[-C, -F];[eye(dim), zeros(dim)]];         % coefficients on current period variables (including expectations of tomorrows variables as separate variables)
g1	= [[L,zeros(dim)];[zeros(dim),eye(dim)]];    % coefficients on last period variables
c   = zeros(2*dim,1);                            % constant
psi = [S;zeros(dim,ns)];                         % coefficients on structural shocks 
pie = [zeros(dim);eye(dim)];                     % coefficients on expectational errors
div = 1;                                     % cutoff between explosive and non-explosive roots

%addpath('gensys')
[NA,NC,NB,fmat,fwt,ywt,gev,eu] = gensys(g0,g1,c,psi,pie,div);
   % [NA,C,NB,q,a,b,z,eu]=gensysct(g0,g1,c,psi,pi,div);
%rmpath('gensys');

%eu

% Extract transition matrix (discard expectational variables that are of no interest here)
A = NA(1:dim,1:dim);
B = NB(1:dim,1:ns);

max_eig = max(abs(eig(A)));
%fprintf('\n Maximum eigenvalue of the system = %g \n',max_eig);

% check solution
%  0 = F*Z_{t+1} + C*Z_t + L*Z_{t-1} + S*shock_t
%  implies that the following must hold:
% F*A^2 + C*A + L

% F*A^2 + C*A + L

residual = abs(F*A^2 + C*A + L);

max_residual = max(max(abs(residual)));

[V, r] = max(residual,[],1);
[VV,c] = max(V);

residual(r(c),c);

max(max(abs(residual)));

% check solution along dynamic path
shock      = zeros(ns,1);
shock(1,1) = 1;
Y0         = randn(dim,1);
Y0(c)      = 100;

dummy=0;
Y          = Y0;  % + B*shock;

for t = 1:10;
    residual = (F*A^2 + C*A + L)*Y;
    max(max(abs(residual)));
    Y = A*Y;
end;


 A = real(NA(1:dim,1:dim));
 B = real(NB(1:dim,:));
 
 res_1 = max(max(abs(F*A^2 + C*A + L)));
 res_2 = max(max(abs(S + (C+F*A)*B)));
 
  max_eigenvalue = max(abs(eig(A)));

% addpath('gensysToAMADist')
%     [NA,NC,NB,fmat,fwt,ywt,gev,eu] = gensysToAMA(g0,g1,c,psi,pie,div,'ama');
% rmpath('gensysToAMADist');

 %addpath('MSV')    
   [NNA,NNB,gev,eu] = msv_one(g0,g1,psi,pie,1.00001);
 %rmpath('MSV')    

 AA = real(NA(1:dim,1:dim));
 BB = real(NB(1:dim,:));

 diff_1 = max(max(abs(AA-A)));
 diff_2 = max(max(abs(BB-B)));
 
 
 
%  
%  A = real(NNA(1:dim,1:dim));
%  B = real(NNB(1:dim,:));
 
% 
% 
% addpath('linlagex')    
%  num_endog = dim;
%  num_exog  = ns;
%  [NA,NB,sorted_eigenvalues,eu]=sp_solve(F,C,L,zeros(dim,ns),S,zeros(ns,ns),dim,num_endog,num_exog,div); %direct to AIM
% rmpath('linlagex')


% % Extract transition matrix (discard expectational variables that are of no interest here)
% A = real(NA(1:dim,1:dim));
% B = real(NB(1:dim,:));
% 
% res_1 = max(max(abs(F*A^2 + C*A + L)))
% 
% res_2 = max(max(abs(S + (C+F*A)*B)))
% 
% 
% max_eig = max(abs(eig(A)));
% 
% 
% fprintf('\n Maximum eigenvalue of the system = %g \n',max_eig);
% 
% 

% 
% 
% 
 [NA,Geigk,NB,K,Mk,OmegaKK,GeigKK,GammaKK,MKK] = fmrss(-C,F,L,S);
 
  AA = real(NA(1:dim,1:dim));
  BB = real(NB(1:dim,:));
  
 res_1 = max(max(abs(F*AA^2 + C*AA + L))); 
 res_2 = max(max(abs(S + (C+F*AA)*BB)));

  
%max(max(abs(A-AA))  )

%max(max(abs(B-BB))  )

max_eigenvalue = max(abs(eig(AA)));

 A=AA;
 B=BB;

