function [H1,H2,F1,F2,V] = solve_value_coop(A0,A1,A2,A3,A4,A5,W,Q,H1,F1,V,beta,t,dampen)

% solves for optimal discretionary policy in the system: 
% A0*Y_{t} = A1*Y_{t-1} + A2*Y_{t+1} + A3*X1_t + A4*X1_{t+1} +A5*V_t
% 
% where: Y is non policy variable
%        X is policy variables
%        V is pure i.i.d. shocks
% 
% policymaker objective given by matrices are W and Q, see Dennis 2007
%
% Solution is of the form:  Y_t = H1*Y_{t-1} + H2*V_t
%                           X_t = F1*Y_{t-1} + F2*V_t
%
%
% INPUT: A0,A1,A2,A3,A4,A5   -   matrices from equilibrium conditions
%        W                   -   loss matrix on nonpolicy variables       
%        Q                   -   loss matrix on policy variables
%        H1,F1               -   starting guesses for solution
%        V                   -   starting guess for value function matrix
%        beta                -   discount factor
%        t                   -   scalar stopping criterion for iteration
%        dampen              -   integer in (0,1), 0 is no dampening
%        
% OUTPUT: H1,H2,F1,F2        -   solution matrices


if size(A0,1)~=size(A0,2); fprintf('\n You must have as many equations as nonpolicy variables \n'); return; end;
if rank(A0) ~= size(A0,2); fprintf('\n Warning your A0 matrix is singular \n'); return; end;

c    = 0;
cmax = 10^6;
diff = 999;

while diff > t
   
  if c > cmax; break; fprintf('\n No convergence after %g iterations \n', cmax); return; end;
   
  c = c + 1;
  V_old   = V;
  H1_old  = H1;
  F1_old  = F1;  
% Step 2: solve for D and P
  D = A0 - A2*H1 - A4*F1;
% Step 3: Update

  Delta_1 =  Q + transpose(A3)*inv(transpose(D))*(W+beta*V)*inv(D)*A3;
  Delta_2 =      transpose(A3)*inv(transpose(D))*(W+beta*V)*inv(D)*A1;
  Delta_3 =      transpose(A3)*inv(transpose(D))*(W+beta*V)*inv(D)*A5;

  F1 = (1-dampen)*(   -inv(Delta_1)*Delta_2  ) + dampen*F1_old;  
  H1 = (1-dampen)*(    inv(D)*(A1+A3*F1)     ) + dampen*H1_old;
  V  = (1-dampen)*(   transpose(H1)*W*H1 + transpose(F1)*Q*F1 + beta*transpose(H1)*V*H1 ) + dampen*V_old;
  
 % if ~all(abs(eig(H1))< 1) == 1; 
 %           fprintf('\n Explosive eigenvalues: better introduce dampening ... \n'); 
 % end;
 
 diff_1 = ([ max(max(abs(H1-H1_old))) ,  max(max(abs(F1-F1_old))), max(max(abs(V-V_old))) ]);
 diff = max(diff_1);
 
 if mod(c,10)==0;
     %fprintf('\n Maximum residual is %g after %g iterations',diff,c)
 end
  
%  colormap(jet);
%  colorbar;
%  subplot(1,3,1);
%  imagesc(abs(H1-H1_old)); colorbar;  title('H1');
%  subplot(1,3,2);
%  imagesc(abs(F1-F1_old)); colorbar;  title('F1');
%  subplot(1,3,3);
%  imagesc(abs(V-V_old));   colorbar;  title('V');
%  pause(1/10000);
  
 
end;  

F2 = -inv(Delta_1)*Delta_3;
H2 =  inv(D)*(A5+A3*F2);

%fprintf('\n Convergence after %g iterations \n', c);

check_1 = -((A0 - A2*H1 -A4*F1)*H1 -A3*F1) + A1;
check_2 = -((A0 - A2*H1)*H2 -A3*F2 - A4*F1*H2) + A5;
  
%fprintf('\n Accuracy check for solving functional equation %g and %g \n', max(max(abs(check_1))), max(max(abs(check_2))) );

  